@@ -1,5 +1,5 @@
 #! /usr/bin/env bash
-$EXTRACTRC `find . -name "*.rc" -o -name "*.ui"` > rc.cpp
+$EXTRACTRC `find . -name "*.rc" -o -name "*.ui"` >> rc.cpp
 $XGETTEXT `find . -name \*.cpp` -o $podir/kpackagekit.pot
 rm -f rc.cpp
 
